import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import './talsec_callback.dart';
import './talsec_config.dart';

export './talsec_callback.dart';
export './talsec_config.dart';

/// Wrapper of general config and general callbacks.
///
/// Sends and receive data from native side, decides which callbacks should be called.
class TalsecApp {
  final TalsecConfig config;
  final TalsecCallback callback;

  const TalsecApp({required this.callback, required this.config});

  /// Setup channels, send data to native side and setup listener from native side
  Future<void> start() async {
    const EventChannel channel = EventChannel('plugins.aheaditec.com/events');
    const MethodChannel configChannel =
        MethodChannel('plugins.aheaditec.com/config');

    /// Android device
    AndroidConfig? androidConfig = config.androidConfig;
    IOSconfig? iosConfig = config.IOSConfig;

    if (Platform.isAndroid && androidConfig != null) {
      configChannel.invokeListMethod('getConfig', {
        'expectedPackageName': androidConfig.expectedPackageName,
        'expectedSigningCertificateHash':
            androidConfig.expectedSigningCertificateHash,
        'watcherMail': config.watcherMail,
        'supportedAlternativeStores': androidConfig.supportedAlternativeStores
      });
    }

    /// iOS device
    else if (Platform.isIOS && iosConfig != null) {
      configChannel.invokeListMethod('getConfig', {
        'appBundleId': iosConfig.appBundleId,
        'appTeamId': iosConfig.appTeamId,
        'watcherMail': config.watcherMail,
      });
    } else {
      throw new FlutterError(
          '${Platform.isAndroid ? "androidConfig" : "IOSconfig"} is not provided.');
    }

    /// Listen to changes from native side
    _setListener(channel);
  }

  /// Receive different events from native side
  void _setListener(EventChannel channel) {
    channel.receiveBroadcastStream().listen((event) {
      switch (event) {

        /// Android: Event received when device is rooted
        case "onRootDetected":
          callback.androidCallback?.onRootDetected?.call();
          break;

        /// Android & iOS: Event received when app is running in debug mode
        case "onDebuggerDetected":
          callback.onDebuggerDetected?.call();
          break;

        /// Android: Event received when app is running on emulator
        case "onEmulatorDetected":
          callback.androidCallback?.onEmulatorDetected?.call();
          break;

        /// Android: Event received when app code integrity is disturbed
        case "onTamperDetected":
          callback.androidCallback?.onTamperDetected?.call();
          break;

        /// Android: Event received when hooking framework is present or used
        case "onHookDetected":
          callback.androidCallback?.onHookDetected?.call();
          break;

        /// Android: Event received when fingerprint is disturbed
        case "onFingerprintDetected":
          callback.androidCallback?.onFingerprintDetected?.call();
          break;

        /// iOS: Event received when app signature does not match expected one
        case "onSignatureDetected":
          callback.IOSCallback?.onSignatureDetected?.call();
          break;

        /// iOS: Event received when jailbreak on device is detected
        case "onJailbreakDetected":
          callback.IOSCallback?.onJailbreakDetected?.call();
          break;

        /// iOS: Event received when app is manipulated on runtime
        case "onRuntimeManipulationDetected":
          callback.IOSCallback?.onRuntimeManipulationDetected?.call();
          break;

        /// iOS: Event received when device is not protected by passcode
        case "onPasscodeDetected":
          callback.IOSCallback?.onPasscodeDetected?.call();
          break;

        /// iOS: Event received when passcode protection of device was changed or removed
        case "onPasscodeChangeDetected":
          callback.IOSCallback?.onPasscodeChangeDetected?.call();
          break;

        /// iOS: Event received when app is running on simulator
        case "onSimulatorDetected":
          callback.IOSCallback?.onSimulatorDetected?.call();
          break;

        /// iOS: Event received when secure enclave layer is missing
        case "onMissingSecureEnclaveDetected":
          callback.IOSCallback?.onMissingSecureEnclaveDetected?.call();
          break;
      }
    });
  }
}
